"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Compound = void 0;
var smath_1 = require("smath");
/**
 * Contains all software used for the calculation of compound values.
 */
var Compound;
(function (Compound_1) {
    /**
     * Represents a compound unit or dimension.
     */
    var Compound = /** @class */ (function () {
        /**
         * Create a new compound unit or dimension.
         * @param exponents Contains an object of exponent values
         * @param toLaTeX A function to convert the exponent to its representation in LaTeX
         */
        function Compound(exponents, toLaTeX) {
            this.exponents = exponents;
            this.toLaTeX = toLaTeX;
            this.num = {};
            this.den = {};
            for (var t in exponents) {
                var exponent = this.getExponent(t);
                if (exponent > 0) {
                    this.num[t] = exponent;
                }
                else if (exponent < 0) {
                    this.den[t] = -exponent;
                }
            }
        }
        /**
         * Combine two compounds by applying a factor on the second compound.
         * @param other Another compount
         * @param factor The factor to use on the other compound
         * @returns The combination of the two compounds
         */
        Compound.prototype.combine = function (other, factor) {
            var exponents_combined = {};
            for (var t in this.exponents) {
                exponents_combined[t] = this.getExponent(t);
            }
            for (var t in other.exponents) {
                exponents_combined[t] = this.getExponent(t) + factor * other.getExponent(t);
            }
            return exponents_combined;
        };
        /**
         * Determine whether two compounds contain the same units or dimensions.
         * @param other Another compound
         * @returns A boolean
         */
        Compound.prototype.is = function (other) {
            var _a;
            var dividend = this.combine(other, -1);
            for (var t in dividend) {
                if (!smath_1.SMath.approx((_a = dividend[t]) !== null && _a !== void 0 ? _a : 0, 0)) {
                    return false;
                }
            }
            return true;
        };
        /**
         * Determine the exponent on the specified unit or dimension.
         * @param exponent The unit or dimension to retrieve the exponent from
         * @returns The exponent
         */
        Compound.prototype.getExponent = function (exponent) {
            var _a;
            return (_a = this.exponents[exponent]) !== null && _a !== void 0 ? _a : 0;
        };
        /**
         * Generate an array of nonzero exponent units or dimensions.
         * @returns An array of nonzero exponent units or dimensions
         */
        Compound.prototype.getNonzeroExponents = function () {
            var nonzeroExponents = [];
            for (var t in this.exponents) {
                if (this.getExponent(t)) {
                    nonzeroExponents.push(t);
                }
            }
            return nonzeroExponents;
        };
        /**
         * Generate LaTeX code for an exponents object.
         * @param expo Any exponents object
         * @returns Partial LaTeX code
         */
        Compound.prototype.prettyPrint = function (expo) {
            var _a;
            var str = '';
            for (var t in expo) {
                if (str.length) {
                    str += ' \\cdot ';
                }
                var exponent = (_a = expo[t]) !== null && _a !== void 0 ? _a : 0;
                if (smath_1.SMath.approx(exponent, 1)) {
                    str += this.toLaTeX(t);
                }
                else if (smath_1.SMath.approx(exponent, 0.5)) {
                    str += '\\sqrt{' + this.toLaTeX(t) + '}';
                }
                else {
                    str += this.toLaTeX(t) + '^{' + exponent.toString() + '}';
                }
            }
            return str;
        };
        /**
         * Generate valid LaTeX code representing this compound.
         * @returns A valid LaTeX equation
         */
        Compound.prototype.toString = function () {
            var str = '';
            var hasNum = Object.keys(this.num).length > 0, hasDen = Object.keys(this.den).length > 0;
            if (hasDen) {
                str += '\\frac{';
            }
            if (hasNum) {
                str += this.prettyPrint(this.num);
            }
            else {
                str += '1';
            }
            if (hasDen) {
                str += '}{' + this.prettyPrint(this.den) + '}';
            }
            return str;
        };
        return Compound;
    }());
    Compound_1.Compound = Compound;
})(Compound || (exports.Compound = Compound = {}));
