"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Conversion = void 0;
var attribute_1 = require("./attribute");
var prefix_1 = require("./prefix");
/**
 * Contains all software used for the calculation of unit conversions.
 */
var Conversion;
(function (Conversion_1) {
    /**
     * Contains information on how units should be converted.
     */
    var Conversion = /** @class */ (function () {
        /**
         * Create a new unit conversion.
         * @param latex The LaTeX representation of this unit.
         * @param scale The scale of this unit in relation to the base units that make up this dimension.
         * @param attribute The physical attribute measured by this unit.
         */
        function Conversion(latex, scale, attribute) {
            this.latex = latex;
            this.scale = scale;
            this.attribute = attribute;
        }
        /**
         * Create a simple unit conversion that defines this unit as the base for its dimension.
         * @param latex The LaTeX representation of this unit.
         * @param exponents The exponents on the base physical dimensions.
         * @returns A simple conversion formula.
         */
        Conversion.simple = function (latex, dimension) {
            return new Conversion(latex, 1, new attribute_1.Attribute.Attribute({}, '', dimension));
        };
        /**
         * Create a new scaled unit using a metric prefix.
         * @param prefix The metric prefix of this scaled unit.
         * @param base The unscaled base unit of this dimension.
         * @returns A scaling conversion formula.
         */
        Conversion.scaled = function (prefix, base) {
            return new Conversion(prefix_1.Prefix.Table[prefix].latex + ' ' + Conversion_1.Table[base]().latex, prefix_1.Prefix.Table[prefix].scale * Conversion_1.Table[base]().scale, Conversion_1.Table[base]().attribute);
        };
        /**
         * Create a new unit that is relative to another.
         * @param latex The LaTeX representation of this unit.
         * @param scale The scale of this unit in relation to the relative unit.
         * @param ref The relative or reference unit.
         * @returns A relative scale formula between this unit and a relative one.
         */
        Conversion.relative = function (latex, scale, ref) {
            return new Conversion(latex, scale * Conversion_1.Table[ref]().scale, Conversion_1.Table[ref]().attribute);
        };
        /**
         * @param latex The LaTeX representation of this unit.
         * @param makeup The equivalent makeup of base units for a unit combined from other units. For example, `kg*m/s^2` for Newtons.
         */
        Conversion.complex = function (latex, makeup) {
            var _a;
            var scale = 1, attr = attribute_1.Attribute.None;
            for (var unit in makeup) {
                var exponent = (_a = makeup[unit]) !== null && _a !== void 0 ? _a : 0;
                scale *= (Math.pow(Conversion_1.Table[unit]().scale, exponent));
                attr = attr.mult(Conversion_1.Table[unit]().attribute, exponent);
            }
            return new Conversion(latex, scale, attr);
        };
        return Conversion;
    }());
    Conversion_1.Conversion = Conversion;
    /**
     * Represents the full conversion table for **absolute** units only.
     */
    Conversion_1.Table = {
        nanosecond: function () { return Conversion.scaled('nano', 'second'); },
        millisecond: function () { return Conversion.scaled('milli', 'second'); },
        second: function () { return Conversion.simple('\\text{s}', 'time'); },
        minute: function () { return Conversion.relative('\\text{min}', 60, 'second'); },
        hour: function () { return Conversion.relative('\\text{h}', 60, 'minute'); },
        day: function () { return Conversion.relative('\\text{d}', 24, 'hour'); },
        week: function () { return Conversion.relative('\\text{w}', 7, 'day'); },
        month: function () { return Conversion.relative('\\text{M}', 1 / 12, 'year'); },
        year: function () { return Conversion.relative('\\text{y}', 365.25, 'day'); },
        nanometer: function () { return Conversion.scaled('nano', 'meter'); },
        micrometer: function () { return Conversion.scaled('micro', 'meter'); },
        micron: function () { return Conversion.scaled('micro', 'meter'); },
        millimeter: function () { return Conversion.scaled('milli', 'meter'); },
        centimeter: function () { return Conversion.scaled('centi', 'meter'); },
        meter: function () { return Conversion.simple('\\text{m}', 'length'); },
        kilometer: function () { return Conversion.scaled('kilo', 'meter'); },
        inch: function () { return Conversion.relative('\\text{in}', 1 / 12, 'foot'); },
        foot: function () { return Conversion.relative('\\text{ft}', 0.3048, 'meter'); },
        yard: function () { return Conversion.relative('\\text{yd}', 3, 'foot'); },
        mile: function () { return Conversion.relative('\\text{mi}', 5280, 'foot'); },
        milligram: function () { return Conversion.scaled('milli', 'gram'); },
        gram: function () { return Conversion.simple('\\text{g}', 'mass'); }, // `kg` is the base unit... but `g` makes things easier.
        kilogram: function () { return Conversion.scaled('kilo', 'gram'); },
        tonne: function () { return Conversion.scaled('mega', 'gram'); },
        ounce: function () { return Conversion.relative('\\text{oz}', 1 / 16, 'pound_mass'); },
        pound_mass: function () { return Conversion.relative('\\text{lb}_{m}', 1 / 2.204623, 'kilogram'); },
        slug: function () { return Conversion.relative('\\text{slug}', 32.174049, 'pound_mass'); },
        stone: function () { return Conversion.relative('\\text{st.}', 14, 'pound_mass'); },
        shortton: function () { return Conversion.relative('\\text{tn}', 2000, 'pound_mass'); },
        milliampere: function () { return Conversion.scaled('milli', 'ampere'); },
        ampere: function () { return Conversion.simple('\\text{A}', 'current'); },
        Kelvin: function () { return Conversion.simple('\\text{K}', 'temperature'); },
        Rankine: function () { return Conversion.relative('\\degree\\text{R}', 5 / 9, 'Kelvin'); },
        Celsius_delta: function () { return Conversion.relative('\\degree\\text{C}_{\\Delta}', 1, 'Kelvin'); },
        Fahrenheight_delta: function () { return Conversion.relative('\\degree\\text{F}_{\\Delta}', 1, 'Rankine'); },
        mole: function () { return Conversion.simple('\\text{mol}', 'substance'); },
        candela: function () { return Conversion.simple('\\text{cd}', 'intensity'); },
        lumen: function () { return Conversion.relative('\\text{lm}', 4 * Math.PI, 'candela'); },
        Newton: function () { return Conversion.complex('\\text{N}', { kilogram: 1, meter: 1, second: -2 }); },
        kiloNewton: function () { return Conversion.scaled('kilo', 'Newton'); },
        pound_force: function () { return Conversion.complex('\\text{lb}_{f}', { slug: 1, foot: 1, second: -2 }); },
        Joule: function () { return Conversion.complex('\\text{J}', { Newton: 1, meter: 1 }); },
        Watt: function () { return Conversion.complex('\\text{W}', { Joule: 1, second: -1 }); },
    };
})(Conversion || (exports.Conversion = Conversion = {}));
