"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Quantity = void 0;
var smath_1 = require("smath");
var unit_1 = require("./unit");
var dimension_1 = require("./dimension");
var attribute_1 = require("./attribute");
/**
 * Contains all software used for the calculation of physical quantities.
 */
var Quantity;
(function (Quantity_1) {
    /**
     * Defines the class for physical quantities.
     */
    var Quantity = /** @class */ (function () {
        /**
         * Create a new instance of a quantity.
         * @param value The value of the quantity
         * @param unit The units attached to the value
         */
        function Quantity(value, unit) {
            this.value = value;
            this.unit = unit;
            this.base = value;
            this.base = smath_1.SMath.expand(value, 0, unit.scale);
        }
        /**
         * Convert this quantity to another set of units.
         * Will throw an error if dimensions do not match.
         * @param newUnit The unit to convert to
         * @returns An equivalent quantity with new units
         */
        Quantity.prototype.as = function (newUnit) {
            if (!this.is(newUnit)) {
                throw new Error('\\text{Dimensions do not match! } ' + this.unit.attribute.dimension.toString() + ' \\text{ vs. } ' + newUnit.attribute.dimension.toString());
            }
            return new Quantity(smath_1.SMath.normalize(this.base, 0, newUnit.scale), newUnit);
        };
        /**
         * Determine whether the dimensions of this quantity matches the input object.
         * @param other Another quantity, unit, attribute, or dimension
         * @returns True if the dimensions match, false otherwise
         */
        Quantity.prototype.is = function (other) {
            if (other instanceof Quantity) {
                return this.unit.attribute.dimension.is(other.unit.attribute.dimension);
            }
            else if (other instanceof unit_1.Unit.Unit) {
                return this.unit.attribute.dimension.is(other.attribute.dimension);
            }
            else if (other instanceof attribute_1.Attribute.Attribute) {
                return this.unit.attribute.dimension.is(other.dimension);
            }
            else if (other instanceof dimension_1.Dimension.Dimension) {
                return this.unit.attribute.dimension.is(other);
            }
            else {
                throw new Error('Cannot compare to ' + (typeof other) + '!');
            }
        };
        /**
         * Multiply this quantity by a constant scalar factor and return the result.
         * @param factor Any scalar value
         * @returns A scaled quantity with the same units
         */
        Quantity.prototype.scale = function (factor) {
            return new Quantity(this.value * factor, this.unit);
        };
        /**
         * Add this quantity to another and return the sum.
         * @param other Another quantity of like dimensions
         * @returns The sum, with the units of the first quantity
         */
        Quantity.prototype.plus = function (other) {
            return new Quantity(this.value + other.as(this.unit).value, this.unit);
        };
        /**
         * Subtract another quantity from this one and return the difference.
         * @param other Another quantity of like dimensions
         * @returns The difference, with the units of the first quantity
         */
        Quantity.prototype.minus = function (other) {
            return this.plus(other.scale(-1));
        };
        /**
         * Multiply this quantity by another and return the product.
         * @param other Another quantity (any dimension)
         * @param exponent An optional exponent to apply on the other quantity
         * @returns The product, with combined units (some may cancel)
         */
        Quantity.prototype.times = function (other, exponent) {
            if (exponent === void 0) { exponent = 1; }
            return new Quantity(this.value * (Math.pow(other.value, exponent)), this.unit.mult(other.unit, exponent));
        };
        /**
         * Divide this quantity by another and return the quotient.
         * @param other Another quantity (any dimension)
         * @param exponent An optional exponent to apply on the other quantity
         * @returns The quotient, with combined units (some may cancel)
         */
        Quantity.prototype.over = function (other, exponent) {
            if (exponent === void 0) { exponent = 1; }
            return this.times(other, -exponent);
        };
        /**
         * Raise this quantity to an exponent and return the power.
         * @param exponent The exponent to raise this unit to
         * @returns The power, with appropriate units
         */
        Quantity.prototype.pow = function (exponent) {
            return new Quantity(Math.pow(this.value, exponent), unit_1.Unit.None.mult(this.unit, exponent));
        };
        /**
         * Generate valid LaTeX code representing this quantity.
         * @returns A valid LaTeX equation
         */
        Quantity.prototype.toString = function () {
            return this.value.toString() + ' \\left[' + this.unit.toString() + '\\right]';
        };
        return Quantity;
    }());
    Quantity_1.Quantity = Quantity;
})(Quantity || (exports.Quantity = Quantity = {}));
