import { Compound } from './compound';
import { Dimension } from './dimension';
/**
 * Contains all software used for the calculation of attributes.
 */
export declare namespace Attribute {
    /**
     * Contains definitions for common attribute names.
     */
    type Name = Dimension.Name | 'area' | 'volume' | 'velocity' | 'acceleration' | 'force' | 'energy' | 'power';
    /**
     * Is an object containing keys of attributes and values of nonzero exponents.
     */
    interface Exponents extends Compound.Exponents<Name> {
    }
    /**
     * Contains information for an attribute.
     */
    class Attribute extends Compound.Compound<Name, Attribute> {
        private readonly latex;
        /**
         * Contains the base dimensions for this attribute.
         */
        readonly dimension: Dimension.Dimension;
        /**
         * Define a new attribute.
         * @param exponents The exponents of the attributes that make up this type
         * @param latex A valid LaTeX equation representing this attributes
         * @param base Determines the base dimension measured by this type (overrides `exponents`)
         */
        constructor(exponents: Exponents, latex?: string, base?: Dimension.Name);
        mult(other: Attribute, exponent: number): Attribute;
        /**
         * Return the common attribute name for this type, or undefined if none exist.
         * @returns The common attribute name, if one exists
         */
        getName(): Name | undefined;
        /**
         * Determine the equivalent attribute type, else return this object.
         * @returns The equivalent attribute type, if one exists
         * @example
         * `mass` * `velocity^2` => `energy`
         * `time` * `time` => `time^2` // no equivalent
         */
        simplify(): Attribute;
        toString(): string;
    }
    /**
     * Represents a dimensionless attribute.
     */
    const None: Attribute;
}
