/**
 * Contains all software used for the calculation of compound values.
 */
export declare namespace Compound {
    /**
     * Represents a type where every object value is a number representing exponents.
     */
    type Exponents<T extends string> = {
        [index in T]?: number;
    };
    /**
     * Represents a compound unit or dimension.
     */
    abstract class Compound<T extends string, C extends Compound<T, C>> {
        private readonly exponents;
        private readonly toLaTeX;
        private readonly num;
        private readonly den;
        /**
         * Create a new compound unit or dimension.
         * @param exponents Contains an object of exponent values
         * @param toLaTeX A function to convert the exponent to its representation in LaTeX
         */
        constructor(exponents: Exponents<T>, toLaTeX: (exponent: T) => string);
        /**
         * Combine two compounds by applying a factor on the second compound.
         * @param other Another compount
         * @param factor The factor to use on the other compound
         * @returns The combination of the two compounds
         */
        protected combine(other: C, factor: number): Exponents<T>;
        /**
         * Multiply this compound by another after applying an exponent on the second compound.
         * @param other Another compound fraction
         * @param exponent The exponent to apply on the other compound
         * @returns The product of the two compounds
         */
        abstract mult(other: C, exponent: number): C;
        /**
         * Determine whether two compounds contain the same units or dimensions.
         * @param other Another compound
         * @returns A boolean
         */
        is(other: C): boolean;
        /**
         * Determine the exponent on the specified unit or dimension.
         * @param exponent The unit or dimension to retrieve the exponent from
         * @returns The exponent
         */
        getExponent(exponent: T): number;
        /**
         * Generate an array of nonzero exponent units or dimensions.
         * @returns An array of nonzero exponent units or dimensions
         */
        getNonzeroExponents(): Array<T>;
        /**
         * Generate LaTeX code for an exponents object.
         * @param expo Any exponents object
         * @returns Partial LaTeX code
         */
        private prettyPrint;
        /**
         * Generate valid LaTeX code representing this compound.
         * @returns A valid LaTeX equation
         */
        toString(): string;
    }
}
