import { Attribute } from './attribute';
import { Dimension } from './dimension';
import { Prefix } from './prefix';
import { Unit } from './unit';
/**
 * Contains all software used for the calculation of unit conversions.
 */
export declare namespace Conversion {
    /**
     * Contains information on how units should be converted.
     */
    export class Conversion {
        readonly latex: string;
        readonly scale: number;
        readonly attribute: Attribute.Attribute;
        /**
         * Create a new unit conversion.
         * @param latex The LaTeX representation of this unit.
         * @param scale The scale of this unit in relation to the base units that make up this dimension.
         * @param attribute The physical attribute measured by this unit.
         */
        constructor(latex: string, scale: number, attribute: Attribute.Attribute);
        /**
         * Create a simple unit conversion that defines this unit as the base for its dimension.
         * @param latex The LaTeX representation of this unit.
         * @param exponents The exponents on the base physical dimensions.
         * @returns A simple conversion formula.
         */
        static simple(latex: string, dimension: Dimension.Name): Conversion;
        /**
         * Create a new scaled unit using a metric prefix.
         * @param prefix The metric prefix of this scaled unit.
         * @param base The unscaled base unit of this dimension.
         * @returns A scaling conversion formula.
         */
        static scaled(prefix: Prefix.Name, base: Unit.Name): Conversion;
        /**
         * Create a new unit that is relative to another.
         * @param latex The LaTeX representation of this unit.
         * @param scale The scale of this unit in relation to the relative unit.
         * @param ref The relative or reference unit.
         * @returns A relative scale formula between this unit and a relative one.
         */
        static relative(latex: string, scale: number, ref: Unit.Name): Conversion;
        /**
         * @param latex The LaTeX representation of this unit.
         * @param makeup The equivalent makeup of base units for a unit combined from other units. For example, `kg*m/s^2` for Newtons.
         */
        static complex(latex: string, makeup: Unit.Exponents): Conversion;
    }
    /**
     * Defines the type for the conversion table.
     */
    type Conversions = {
        [index in Unit.Name]: () => Conversion;
    };
    /**
     * Represents the full conversion table for **absolute** units only.
     */
    export const Table: Conversions;
    export {};
}
