import { Unit } from './unit';
import { Dimension } from './dimension';
import { Attribute } from './attribute';
/**
 * Contains all software used for the calculation of physical quantities.
 */
export declare namespace Quantity {
    /**
     * Defines the class for physical quantities.
     */
    class Quantity {
        readonly value: number;
        readonly unit: Unit.Unit;
        /**
         * The value converted to base units
         */
        private readonly base;
        /**
         * Create a new instance of a quantity.
         * @param value The value of the quantity
         * @param unit The units attached to the value
         */
        constructor(value: number, unit: Unit.Unit);
        /**
         * Convert this quantity to another set of units.
         * Will throw an error if dimensions do not match.
         * @param newUnit The unit to convert to
         * @returns An equivalent quantity with new units
         */
        as(newUnit: Unit.Unit): Quantity;
        /**
         * Determine whether the dimensions of this quantity matches the input object.
         * @param other Another quantity, unit, attribute, or dimension
         * @returns True if the dimensions match, false otherwise
         */
        is(other: Quantity | Unit.Unit | Attribute.Attribute | Dimension.Dimension): boolean;
        /**
         * Multiply this quantity by a constant scalar factor and return the result.
         * @param factor Any scalar value
         * @returns A scaled quantity with the same units
         */
        scale(factor: number): Quantity;
        /**
         * Add this quantity to another and return the sum.
         * @param other Another quantity of like dimensions
         * @returns The sum, with the units of the first quantity
         */
        plus(other: Quantity): Quantity;
        /**
         * Subtract another quantity from this one and return the difference.
         * @param other Another quantity of like dimensions
         * @returns The difference, with the units of the first quantity
         */
        minus(other: Quantity): Quantity;
        /**
         * Multiply this quantity by another and return the product.
         * @param other Another quantity (any dimension)
         * @param exponent An optional exponent to apply on the other quantity
         * @returns The product, with combined units (some may cancel)
         */
        times(other: Quantity, exponent?: number): Quantity;
        /**
         * Divide this quantity by another and return the quotient.
         * @param other Another quantity (any dimension)
         * @param exponent An optional exponent to apply on the other quantity
         * @returns The quotient, with combined units (some may cancel)
         */
        over(other: Quantity, exponent?: number): Quantity;
        /**
         * Raise this quantity to an exponent and return the power.
         * @param exponent The exponent to raise this unit to
         * @returns The power, with appropriate units
         */
        pow(exponent: number): Quantity;
        /**
         * Generate valid LaTeX code representing this quantity.
         * @returns A valid LaTeX equation
         */
        toString(): string;
    }
}
